<?php
/**
 * Suprimentos\Logic\Comprador
 *
 * @category Php
 * @package  Suprimentos\Logic\Comprador
 * @author   joaocorte <joao.corte@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Suprimentos\Logic\Comprador;

use Suprimentos\Logic\Exception\ExceptionCotacao;
use Suprimentos\Entities\Comproper;
use Geral\Entities\Parasupri;
use Geral\Entities\Comprador;
use Geral\Util\Ambiente;

/**
 * Class CompradorLogic
 *
 * @category Php
 * @package  Suprimentos\Logic\Comprador
 * @author   joaocorte <joao.corte@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class CompradorLogic {

    const CDOPERADOR = 'CDOPERADOR';
    const IDUMPRODCMP = 'idumprodcmp';

    public static function validaExclusaoCotacao($nrorg)
    {
        $cdcomprador = tk_get_field_value(self::CDOPERADOR);
        $cdoperador  = Ambiente::getCdOperador();

        /**
         * Comprador relacionado
         * @var \Suprimentos\Entities\Comproper $cdcompraoper
         */
        $cdcompraoper = Comproper::getObjectsByWhere(
            'NRORG = ? AND CDCOMPRADOR = ?',
            array($nrorg, $cdcomprador)
        );

        if (!$cdcompraoper){

            /**
             * Comprador
             * @var \Geral\Entities\Comprador $comprador
             */
            $comprador= Comprador::getObjectByWhere(
                'NRORG = ? AND CDOPERADOR = ?',
                array($nrorg, $cdoperador)
            );

            if (!$comprador){
                ExceptionCotacao::operadorNaoComprador();
            } else {
                $cdfilial = Ambiente::getCdFilial();

                /**
                 * Parametros
                 * @var \Geral\Entities\Parasupri $parasupri
                 */
                $parasupri = Parasupri::getObjectByWhere(
                    'NRORG = ? AND CDFILIAL = ?',
                    array($nrorg, $cdfilial)
                );
                $parasupri = $parasupri->getIdumprodcmp();
                if ($parasupri != 'S'){
                    $comprador = $comprador->getCdoperador();
                    if ($cdoperador != $comprador){
                        ExceptionCotacao::cotacaoNaoPertenceComprador();
                    }
                }
            }
        } else {
            $operadorelacionado = 'N';

            /**
             * @var \Suprimentos\Entities\Comproper $operrelac
             */
            foreach ($cdcompraoper as $operrelac){

                if ($operrelac->getCdoperador() == $cdoperador){
                    $operadorelacionado = 'S';
                    if ($operrelac->getIdcotcmpoper() == 'N'){
                        ExceptionCotacao::operadorSemAutorizacao();
                    }
                }

            }
            if ($operadorelacionado == 'N'){
                ExceptionCotacao::operadorNaoRelacComprador();
            }
        }
    }
} 