create or replace FUNCTION FCT_ESTIMADO_MAPAFATURA(P_CDSERVORCA IN FCT_SERVICOORCA.CDSERVORCA%TYPE,
                                                P_TIPO IN VARCHAR2,
                                                P_DTPERIODO IN VARCHAR2,
                                                P_NRORG IN FCT_CONTRATO.NRORG%TYPE)

RETURN FCT_ESTIMADO_MAPAFATURA_TABLE PIPELINED
IS
OUT_RECORD FCT_ESTIMADO_MAPAFATURA_TYPE := FCT_ESTIMADO_MAPAFATURA_TYPE (NULL, NULL, NULL, NULL, NULL,
                                                                   NULL, NULL);
V_SERVICOANTE          NUMBER;
V_PARCEIROANTE         NUMBER;
V_CONTRATOANTE         NUMBER;
V_VALORAUX             NUMBER;
V_VALORSERV            NUMBER;
V_VALORPARC            NUMBER;
V_VALORCONTR           NUMBER;
V_DESCONTONEG          NUMBER;
V_DESCPERIODO          NUMBER;
V_QTPARCELA            NUMBER;
V_IDTIPONEGOCIA        VARCHAR2(2);
V_VRPARCELANEGOCIA     NUMBER;
V_IDREALIZARNEGOCIACAO VARCHAR2(1);

BEGIN

	V_PARCEIROANTE := -1;
    V_CONTRATOANTE := -1;

	V_VALORAUX     := 0;
	V_VALORSERV    := 0;
    V_VALORPARC    := 0;
    V_VALORCONTR   := 0;


	FOR C_CALC IN (SELECT FAT.NRPARCFATURA, CONT.NRSEQCONTRATO, ITEM.NRITEMOCONTRATO, ITCONT.CDSERVORCA, FAT.NRFATURACONT, 
                          ITCONT.NMITEMCONTRATO, ITCONT.VRTOTITEMCONT 
					 FROM FCT_FATURACONT FAT
					 JOIN FCT_ITFATURACONT ITEM
					   ON ITEM.NRFATURACONT = FAT.NRFATURACONT 
					  AND ITEM.NRORG = FAT.NRORG
					 JOIN FCT_ITEMCONTRATO ITCONT
					   ON ITCONT.NRITEMOCONTRATO = ITEM.NRITEMOCONTRATO
					  AND ITCONT.NRORG = ITEM.NRORG
					 JOIN FCT_CONTRATO CONT
					   ON CONT.NRSEQCONTRATO = ITCONT.NRSEQCONTRATO
					  AND CONT.NRORG = ITCONT.NRORG
					WHERE TO_CHAR(FAT.DTFINCOMPET , 'MM/YYYY') = P_DTPERIODO
					  AND ITCONT.CDSERVORCA = P_CDSERVORCA
					  AND FAT.NRORG = P_NRORG
			     ORDER BY FAT.NRPARCFATURA, CONT.NRSEQCONTRATO) LOOP


		V_VALORAUX := C_CALC.VRTOTITEMCONT;

		/* 1 - Calcula o desconto da negociao de valores (negociao e item)*/
		BEGIN
			SELECT DECODE(NVL(ITNEG.VRPERCDESCITEM,0), 0, NEG.VRPERCDESCITEM, ITNEG.VRPERCDESCITEM)
			  INTO V_DESCONTONEG
			  FROM FCT_NEGOCIACAOCONTRATO NEG
			  JOIN FCT_ITNEGOCIACAOCONTRATO ITNEG
				ON ITNEG.NRNEGOCIACAOCONTRATO = NEG.NRNEGOCIACAOCONTRATO
			   AND ITNEG.NRORG = NEG.NRORG
			 WHERE ITNEG.NRITEMOCONTRATO = C_CALC.NRITEMOCONTRATO
			   AND NEG.NRPARCFATURA = C_CALC.NRPARCFATURA
			   AND NEG.NRSEQCONTRATO = C_CALC.NRSEQCONTRATO
			   AND NEG.NRORG = P_NRORG;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_DESCONTONEG := 0;
		END;

		IF (V_DESCONTONEG > 0) THEN
			V_VALORAUX := V_VALORAUX - ((V_VALORAUX*V_DESCONTONEG)/100);
		END IF;

		/* 2 - Calcula o desconto por perodo */
		BEGIN
			SELECT DESCPER.VRPERCDESCITEM
			  INTO V_DESCPERIODO
		      FROM FCT_DESCPERIODONEGOCONT DESCPER
		      JOIN FCT_NEGOCIACAOCONTRATO NEG
		        ON NEG.NRNEGOCIACAOCONTRATO = DESCPER.NRNEGOCIACAOCONTRATO
			   AND NEG.NRORG = DESCPER.NRORG
			  JOIN FCT_ITNEGOCIACAOCONTRATO ITNEG
				ON ITNEG.NRNEGOCIACAOCONTRATO = NEG.NRNEGOCIACAOCONTRATO
			   AND ITNEG.NRORG = NEG.NRORG
			 WHERE ITNEG.NRITEMOCONTRATO = C_CALC.NRITEMOCONTRATO
			   AND NEG.NRPARCFATURA = C_CALC.NRPARCFATURA
			   AND NEG.NRSEQCONTRATO = C_CALC.NRSEQCONTRATO
			   AND NEG.NRORG = P_NRORG
			   AND DTFINALVALIDADEDESC >= LAST_DAY(TO_DATE('01/'||P_DTPERIODO))
			   AND DTFINALVALIDADEDESC IS NOT NULL
			   AND DTFINALVALIDADEDESC = (SELECT MIN(DTFINALVALIDADEDESC)
											FROM FCT_DESCPERIODONEGOCONT DESCPER
											JOIN FCT_NEGOCIACAOCONTRATO NEG
											  ON NEG.NRNEGOCIACAOCONTRATO = DESCPER.NRNEGOCIACAOCONTRATO
											 AND NEG.NRORG = DESCPER.NRORG
											JOIN FCT_ITNEGOCIACAOCONTRATO ITNEG
											  ON ITNEG.NRNEGOCIACAOCONTRATO = NEG.NRNEGOCIACAOCONTRATO
											  AND ITNEG.NRORG = NEG.NRORG
											WHERE ITNEG.NRITEMOCONTRATO = C_CALC.NRITEMOCONTRATO
											  AND NEG.NRPARCFATURA = C_CALC.NRPARCFATURA
											  AND NEG.NRSEQCONTRATO = C_CALC.NRSEQCONTRATO
											  AND NEG.NRORG = P_NRORG
											  AND DTFINALVALIDADEDESC >= LAST_DAY(TO_DATE('01/'||P_DTPERIODO))
											  AND DTFINALVALIDADEDESC IS NOT NULL);
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_DESCPERIODO := 0;
		END;

		IF (V_DESCPERIODO > 0) THEN
			V_VALORAUX := V_VALORAUX - ((V_VALORAUX*V_DESCPERIODO)/100);
		END IF;

		/* 3 - Se houver parcelamento, calcula o valor do parcelamento */
		BEGIN
			SELECT MAX(PARCELA.NRPARCELA) AS QTPARCELA
			INTO V_QTPARCELA 
			FROM FCT_PARCELANEGOCONT PARCELA
			JOIN FCT_NEGOCIACAOCONTRATO NEG
			  ON NEG.NRNEGOCIACAOCONTRATO = PARCELA.NRNEGOCIACAOCONTRATO
			 AND NEG.NRORG = PARCELA.NRORG
			JOIN FCT_ITNEGOCIACAOCONTRATO ITNEG
			  ON ITNEG.NRNEGOCIACAOCONTRATO = NEG.NRNEGOCIACAOCONTRATO
			 AND ITNEG.NRORG = NEG.NRORG
		   WHERE ITNEG.NRITEMOCONTRATO = C_CALC.NRITEMOCONTRATO
			 AND NEG.NRPARCFATURA = C_CALC.NRPARCFATURA
			 AND NEG.NRSEQCONTRATO = C_CALC.NRSEQCONTRATO
			AND NEG.NRORG = P_NRORG;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_QTPARCELA := 0;
		END;

		IF (V_QTPARCELA > 0) THEN
			V_VALORAUX := V_VALORAUX/V_QTPARCELA;
		END IF;


		/* 4 - Se houver negociao no contrato, calcula a negociao do contrato */

		BEGIN
			SELECT IDTIPONEGOCIA, VRPARCELANEGOCIA
			  INTO V_IDTIPONEGOCIA, V_VRPARCELANEGOCIA
			  FROM (SELECT NEGCONT.IDTIPONEGOCIA, PARCNEGCT.VRPARCELANEGOCIA
					  FROM FCT_NEGCONTRATO NEGCONT
					  JOIN FCT_PARCELASNEGCONTR PARCNEGCT
					    ON PARCNEGCT.NRNEGCONTRATO = NEGCONT.NRNEGCONTRATO
					   AND PARCNEGCT.NRORG = NEGCONT.NRORG
			 		 WHERE NEGCONT.IDREALIZARNEGOCIACAO = 'M'
					   AND ((NEGCONT.NRSEQCONTRATO = C_CALC.NRSEQCONTRATO AND NEGCONT.NRAGRUPAMENTO IS NULL) 
						    OR (NEGCONT.NRSEQCONTRATO IS NULL AND NEGCONT.NRAGRUPAMENTO = (SELECT NRAGRUPAMENTO 
						 																	 FROM FCT_RELACAGRUPAMENTOCONT
																						    WHERE NRSEQCONTRATO = C_CALC.NRSEQCONTRATO))) 
					   AND TO_DATE('01/'||P_DTPERIODO) BETWEEN DTINIPARCNEGOCIA AND DTFIMPARCNEGOCIA
					   AND NEGCONT.NRORG = P_NRORG

            UNION ALL

            SELECT  NEGCONT.IDTIPONEGOCIA, PARCNEGCT.VRPARCELANEGOCIA
			  FROM FCT_NEGCONTRATO NEGCONT
			  JOIN FCT_PARCELASAPURNEGCONTRATO PARCNEGCT
				ON PARCNEGCT.NRNEGCONTRATO = NEGCONT.NRNEGCONTRATO
			   AND PARCNEGCT.NRORG = NEGCONT.NRORG
			 WHERE NEGCONT.IDREALIZARNEGOCIACAO = 'A'
			   AND ((NEGCONT.NRSEQCONTRATO = C_CALC.NRSEQCONTRATO AND NEGCONT.NRAGRUPAMENTO IS NULL) 
					OR (NEGCONT.NRSEQCONTRATO IS NULL AND NEGCONT.NRAGRUPAMENTO = (SELECT NRAGRUPAMENTO 
                                                                                     FROM FCT_RELACAGRUPAMENTOCONT
																					WHERE NRSEQCONTRATO = C_CALC.NRSEQCONTRATO)))
			   AND PARCNEGCT.NRFATURACONT = C_CALC.NRFATURACONT
			   AND NEGCONT.NRORG = P_NRORG);

		EXCEPTION
			WHEN NO_DATA_FOUND THEN BEGIN
				V_IDTIPONEGOCIA        := NULL;
				V_VRPARCELANEGOCIA     := NULL;
			END;
		END;

		IF V_IDTIPONEGOCIA IS NOT NULL THEN

			IF V_IDTIPONEGOCIA = 'VF' THEN -- Valor Fixo
				V_VALORAUX := V_VRPARCELANEGOCIA;
            END IF;
			IF V_IDTIPONEGOCIA = 'DP' THEN -- Desconto (Percentual)
				V_VALORAUX := V_VALORAUX - ((V_VALORAUX*V_VRPARCELANEGOCIA)/100);
			END IF;
            IF V_IDTIPONEGOCIA = 'DV' THEN  -- Desconto (Valor)
				V_VALORAUX := V_VALORAUX - V_VRPARCELANEGOCIA;
			END IF;
            IF V_IDTIPONEGOCIA = 'AP' THEN -- Acrscimo (Percentual)
				V_VALORAUX := V_VALORAUX + ((V_VALORAUX*V_VRPARCELANEGOCIA)/100);
			END IF;
            IF V_IDTIPONEGOCIA = 'AV' THEN-- Acrscimo (Valor)
				V_VALORAUX := V_VALORAUX + V_VRPARCELANEGOCIA;
			END IF;

		END IF;


		/* 5 - Grava os registros */

		IF (P_TIPO = 'SERVICO') THEN

			V_VALORSERV := V_VALORSERV + V_VALORAUX; 

		END IF;


		IF (P_TIPO = 'PARCEIRO') THEN


			IF ((V_PARCEIROANTE <> -1) AND (C_CALC.NRPARCFATURA <> V_PARCEIROANTE)) THEN 

				OUT_RECORD.CDSERVORCA    := P_CDSERVORCA;
				OUT_RECORD.NRPARCNEGOCIO := V_PARCEIROANTE;
				OUT_RECORD.NRSEQCONTRATO := NULL;
				OUT_RECORD.PERIODO       := P_DTPERIODO;
				OUT_RECORD.VRPRECO       := V_VALORPARC;
				OUT_RECORD.NRORG         := P_NRORG;
				OUT_RECORD.DTINCLUSAO    := SYSDATE;

				PIPE ROW (OUT_RECORD);
                
                
                V_PARCEIROANTE := C_CALC.NRPARCFATURA;
				V_VALORPARC := V_VALORAUX;
                
			ELSE 

				V_PARCEIROANTE := C_CALC.NRPARCFATURA;
				V_VALORPARC := V_VALORPARC + V_VALORAUX; 

			END IF;

		END IF;

		IF (P_TIPO = 'CONTRATO') THEN

			IF ((V_CONTRATOANTE <> -1) AND (C_CALC.NRSEQCONTRATO <> V_CONTRATOANTE)) THEN 
				OUT_RECORD.CDSERVORCA    := P_CDSERVORCA;
				OUT_RECORD.NRPARCNEGOCIO := V_PARCEIROANTE;
				OUT_RECORD.NRSEQCONTRATO := V_CONTRATOANTE;
				OUT_RECORD.PERIODO       := P_DTPERIODO;
				OUT_RECORD.VRPRECO       := V_VALORCONTR;
				OUT_RECORD.NRORG         := P_NRORG;
				OUT_RECORD.DTINCLUSAO    := SYSDATE;

				PIPE ROW (OUT_RECORD);
                
                V_CONTRATOANTE := C_CALC.NRSEQCONTRATO;
				V_PARCEIROANTE := C_CALC.NRPARCFATURA;
				V_VALORCONTR :=  V_VALORAUX; 

			ELSE

				V_CONTRATOANTE := C_CALC.NRSEQCONTRATO;
				V_PARCEIROANTE := C_CALC.NRPARCFATURA;
				V_VALORCONTR := V_VALORCONTR + V_VALORAUX; 

			END IF;

		END IF;

	END LOOP;

	IF (P_TIPO = 'SERVICO') THEN

		OUT_RECORD.CDSERVORCA    := P_CDSERVORCA;
		OUT_RECORD.NRPARCNEGOCIO := NULL;
		OUT_RECORD.NRSEQCONTRATO := NULL;
		OUT_RECORD.PERIODO       := P_DTPERIODO;
		OUT_RECORD.VRPRECO       := V_VALORSERV;
		OUT_RECORD.NRORG         := P_NRORG;
		OUT_RECORD.DTINCLUSAO    := SYSDATE;

		PIPE ROW (OUT_RECORD);

	END IF;


	IF (P_TIPO = 'PARCEIRO') THEN

		OUT_RECORD.CDSERVORCA    := P_CDSERVORCA;
		OUT_RECORD.NRPARCNEGOCIO := V_PARCEIROANTE;
		OUT_RECORD.NRSEQCONTRATO := NULL;
		OUT_RECORD.PERIODO       := P_DTPERIODO;
		OUT_RECORD.VRPRECO       := V_VALORPARC;
		OUT_RECORD.NRORG         := P_NRORG;
		OUT_RECORD.DTINCLUSAO    := SYSDATE;

		PIPE ROW (OUT_RECORD);

	END IF;

	IF (P_TIPO = 'CONTRATO') THEN

		OUT_RECORD.CDSERVORCA    := P_CDSERVORCA;
		OUT_RECORD.NRPARCNEGOCIO := V_PARCEIROANTE;
		OUT_RECORD.NRSEQCONTRATO := V_CONTRATOANTE;
		OUT_RECORD.PERIODO       := P_DTPERIODO;
		OUT_RECORD.VRPRECO       := V_VALORCONTR;
		OUT_RECORD.NRORG         := P_NRORG;
		OUT_RECORD.DTINCLUSAO    := SYSDATE;

		PIPE ROW (OUT_RECORD);

	END IF;

	RETURN;

END;

/
